function M = computeGrains2(grains,xvec,yvec,windowSizeX,windowSizeY,lowtrunc,exponent,ds)

% M = computeGrains(grains,xvec,yvec,windowSize,lowtrunc,exponent,ds)
%
% inputs:
%   grains: [id x y minoraxis majoraxis eccentricity] (output from
%    identifygrains.m)
%   xvec: vector of x-location values used as center points of a grid
%   yvec: vector of y-location values used as center points of a grid
%   windowSize: size of the window used to grab grains for spatial
%    distributions (should be >= the grid size -- the spacing within xvec
%    and yvec)
%       note: computeGrains2 splits windowSize into x and y components in
%             case dx and dy aren't the same size. (9/23/2010)
%   lowtrunc: lower truncation value for computing distributions (mm)
%   exponent: exponent used to convert the  area-by-number gsd
%    from the image processing to something else. Typical exponents are:
%    volume by weight to area by number: -2; to area by weight: 1
%    area by number to volume by weight: 2; to area by weight: 3
%    area by weight to volume by weight: -1; to area by number: -3
%    note that grid-by-number is equivalent to volume-by-weight
%    set exponent to 0 if no conversion desired
%   ds: vector of grain size classes desired for the cdf distributions
%
% output:
%   M: structure containing the following:
%       M.x: nx by ny matrix of x-locations of grid centerpoints
%       M.y: nx by ny matrix of y-locations of grid centerpoints
%       M.pdf: nx by ny by nd matrix of pdf of gsd, by grain size class
%       M.cdf: nx by ny by (nd+1) matrix of cdf of gsd by grain size class
%       M.di: nd vector of the pdf grain size classes (mm)
%       M.ds: (nd+1) vector of the cdf grain size classes (mm)
%       M.D50: nx by ny matrix of local D50 (mm)
%       M.D16: nx by ny matrix of local D16 (mm)
%       M.D84: nx by ny matrix of local D84 (mm)
%       M.Dg: nx by ny matrix of local geometric mean (mm)
%       M.sig: nx by ny matrix of local geometric standard deviation (mm?)
%       M.numGrains: nx by ny matrix of number of grains used to generate
%        the pdf/cdf
%       M.rawData: nx by ny cell array of raw grains (same format as grains
%        input) used to make the distributions


halfwidthX = windowSizeX/2;
halfwidthY = windowSizeY/2;
nx = length(xvec);
ny = length(yvec);
nd = length(ds)-1;

%initialize M
M.x = zeros(ny,nx);
M.y = zeros(ny,nx);
M.pdf = zeros(ny,nx,nd);
M.cdf = zeros(ny,nx,nd+1);
M.di = zeros(nd,1);
M.ds = zeros(nd+1,1);
M.D50 = zeros(ny,nx);
M.D16 = zeros(ny,nx);
M.D84 = zeros(ny,nx);
M.Dg = zeros(ny,nx);
M.sig = zeros(ny,nx);
M.numGrains = zeros(ny,nx);
% M.rawData = cell(ny,nx);
M.ds_full = cell(ny,nx);
M.cdf_full = cell(ny,nx);

% Fill in M.di and M.ds
M.ds = ds;
di = zeros(nd,1);
for m=1:nd
    di(m) = sqrt(ds(m)*ds(m+1));
end
M.di = di;
psis = log2(ds);

totalcalcs = length(xvec)*length(yvec);

h=waitbar(0,'calculating gsds');
jj=1;
for i=1:length(xvec)
    for j=1:length(yvec)
        xcenter = xvec(i);
        ycenter = yvec(j);
        xmin = xcenter - halfwidthX;
        xmax = xcenter + halfwidthX;
        ymin = ycenter - halfwidthY;
        ymax = ycenter + halfwidthY;
        
        inds = find(grains(:,2) >= xmin & grains(:,2) <= xmax ...
            & grains(:,3) >= ymin & grains(:,3) <= ymax ...
            & grains(:,4) > 0);
        
        grainstemp = grains(inds,:);
        [D, cdf, D16, D50, D84] = analyzegrains(grainstemp,lowtrunc,exponent);
        %D and cdf are the grains (not classfied into ds classes). It
        %should essentially be the number of grains greater than the
        %truncation size, with the possible exception of the case where
        %more than one grain have EXACTLY (within 0.001 mm) the same
        %b-axis.
        
        M.x(j,i) = xcenter;
        M.y(j,i) = ycenter;
        M.D16(j,i) = D16;
        M.D50(j,i) = D50;
        M.D84(j,1) = D84;
        M.Dg(j,i) = calcDg(D',cdf);
        M.sig(j,i) = calcSig(D',cdf);
        
        % save the pdf and cdf
        M.ds_full{j,i} = D';
        M.cdf_full{j,i} = cdf;
        
        psi_full = log2(D');
        cdf_interp = interp1(psi_full,cdf,psis);
        for k=1:length(ds)
            if ds(k) >= max(D)
                cdf_interp(k) = 100;
            end
        end
        
        M.cdf(j,i,:) = cdf_interp;
        [Di,pdf] = cdf2pdf(ds,cdf_interp);
        if Di == di
            M.pdf(j,i,:) = pdf;
        else
            fprintf('M.pdf(%i,%i,:) not completed since di not equal to Di\n',j,i);
        end
        
        M.numGrains(j,i) = length(D);
        
        % save the grains data
%         M.rawData{j,i} = [
        
        
        waitbar(jj/totalcalcs);
        jj=jj+1;
    end
end

close(h);
        