function [M, xvec, yvec] = createM(grainsmatfile, xmin, xmax, ymin, ymax, dx, dy, ds,...
    wRatio, toy, lowtrunc, exponent)
%
% calls computeGrains to create M structure.
% makes the M structure by
% reading in grain data and putting it on a grid and then, optionally, b)
% uses the data to construct a toy model with three patches.
%
% required inputs:
%   grainsmatfile: mat file containing output from analyzegrains (I think).
%      should have only one variable in it with 'grain' in its name, and
%      the structure of that variable should be [id x y minoraxis majoraxis
%      eccentricity].
%   xmin, xmax: min and max x-values for the grid (meters)
%   ymin, ymax: min and max y-values for the grid (meters)
%   dx: grid spacing in x direction (meters)
%   dy: grid spacing in y direction (meters)
%   ds: vector of grain size classes for cdf (mm)
% optional inputs:
%   wRatio: scaling of window from which grains are pulled to make gsds
%      (can be greater than or equal to max(dx,dy)). [1]
%   toy: set to 1 to make the toy example [0]
%   lowtrunc: lower truncation level of gsds (mm) [4]
%   exponent: exponent for gsd conversion [2]
% output:
%   M: big structure with all of the grain size data conveniently organized


switch nargin
    case 8
        wRatio = 1;
        toy = 0;
        lowtrunc = 4;
        exponent = 2;
    case 9
        toy = 0;
        lowtrunc = 4;
        exponent = 2;
    case 10
        lowtrunc = 4;
        exponent = 2;
    case 11
        exponent = 2;
end

if nargin < 8 || nargin > 12
    error('incorrect number of inputs to createM\n')
end

if wRatio < 1
    error('wRatio must be >= 1!\n')
end

% the mat file will have one variable that has 'grain' in it.
z = load(grainsmatfile); %makes a struct
if length(z) > 1
    error('matfile has too many variables!\n')
end

b = fieldnames(z);

a = getfield(z,b{1}); %now a is our grains variable

xvec = [xmin:dx:xmax];
yvec = [ymin:dy:ymax];

% nx = length(xvec);
% ny = length(yvec);
% numNodes = nx*ny;

windowSizeX = wRatio * dx;
windowSizeY = wRatio * dy;


%set windowsize to dx here
M = computeGrains2(a,xvec,yvec,windowSizeX,windowSizeY,lowtrunc,exponent,ds);


%% make a toy example
if toy == 1;
    
    [ny,nx,nd] = size(M.pdf);
    [r1,c1] = find(M.D50 == min(min(M.D50)));
    [r2,c2] = find(M.D50 == max(max(M.D50)));
    M.pdf_orig = M.pdf;
    M.pdf = repmat(M.pdf_orig(r2,c2,:),[ny nx 1]);
    M.cdf_orig = M.cdf;
    M.cdf = repmat(M.cdf_orig(r2,c2,:),[ny nx 1]);
    [r3,c3] = find(M.x >= 20 & M.x <= 23 & M.y >= 1 & M.y <= 2);
    [r4,c4] = find(M.x >= 30 & M.x <= 34 & M.y >= 1.5 & M.y <= 2.1);
    for i=1:length(r3)
        M.pdf(r3(i),c3(i),:) = M.pdf_orig(r1,c1,:);
        M.cdf(r3(i),c3(i),:) = M.cdf_orig(r1,c1,:);
    end
    for i=1:length(r4)
        M.pdf(r4(i),c4(i),:) = M.pdf_orig(r1,c1,:);
        M.cdf(r4(i),c4(i),:) = M.cdf_orig(r1,c1,:);
    end
    M.D50_orig = M.D50;
    M.D50 = repmat(M.D50_orig(r2,c2,:),[ny nx 1]);
    for i=1:length(r3)
        M.D50(r3(i),c3(i),:) = M.D50_orig(r1,c1,:);
    end
    for i=1:length(r4)
        M.D50(r4(i),c4(i),:) = M.D50_orig(r1,c1,:);
    end
    M.Dg_orig = M.Dg;
    M.Dg = repmat(M.Dg_orig(r2,c2,:),[ny nx 1]);
    for i=1:length(r3)
        M.Dg(r3(i),c3(i),:) = M.Dg_orig(r1,c1,:);
    end
    for i=1:length(r4)
        M.Dg(r4(i),c4(i),:) = M.Dg_orig(r1,c1,:);
    end
    M.sig_orig = M.sig;
    M.sig = repmat(M.sig_orig(r2,c2,:),[ny nx 1]);
    for i=1:length(r3)
        M.sig(r3(i),c3(i),:) = M.sig_orig(r1,c1,:);
    end
    for i=1:length(r4)
        M.sig(r4(i),c4(i),:) = M.sig_orig(r1,c1,:);
    end

end